==> apace-0.0.1.tar.gz: apace-0.0.1/apace/utils.py <==
import collections


class Signal:
    def __init__(self, *args):
        self.callbacks = set()
        for signal in args:
            signal.register(self)

    def __call__(self, *args, **kwargs):
        for callback in self.callbacks:
            callback(*args, **kwargs)

    def register(self, callback):
        self.callbacks.add(callback)


class Flag:
    def __init__(self, initial_value, signals=None):
        self.value = initial_value
        for signal in signals:
            signal.register(self.set_value(True))

    def __bool__(self):
        return self.value

    def set_value(self, value):
        self.value = value


def flatten(iterable):
    for element in iterable:
        if isinstance(element, collections.Iterable) and not isinstance(element, (str, bytes)):
            yield from flatten(element)
        else:
            yield element


class AttrDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__dict__ = self


class Structure:
    pass


# based on https://github.com/apieum/weakreflist/blob/master/weakreflist/weakreflist.py
# modified
from weakref import ref, ReferenceType
import sys


def is_slice(index):
    return isinstance(index, slice)


class WeakList(list):

    def __init__(self, items=list()):
        list.__init__(self, self._refs(items))

    def value(self, item):
        return item() if isinstance(item, ReferenceType) else item

    def ref(self, item):
        try:
            item = ref(item, self.remove_all)
        finally:
            return item

    def __contains__(self, item):
        return list.__contains__(self, self.ref(item))

    def __getitem__(self, index):
        items = list.__getitem__(self, index)
        return type(self)(self._values(items)) if is_slice(index) else self.value(items)

    def __setitem__(self, index, item):
        items = self._refs(item) if is_slice(index) else self.ref(item)
        return list.__setitem__(self, index, items)

    def __iter__(self):
        return iter(self[index] for index in range(len(self)))

    def __reversed__(self):
        reversed_self = type(self)(self)
        reversed_self.reverse()
        return reversed_self

    # modified
    def clear(self):
        del self[:]

    def append(self, item):
        list.append(self, self.ref(item))

    def remove(self, item):
        return list.remove(self, self.ref(item))

    def remove_all(self, item):
        item = self.ref(item)
        while list.__contains__(self, item):
            list.remove(self, item)

    def index(self, item):
        return list.index(self, self.ref(item))

    def count(self, item):
        return list.count(self, self.ref(item))

    def pop(self, index=-1):
        return self.value(list.pop(self, self.ref(index)))

    def insert(self, index, item):
        return list.insert(self, index, self.ref(item))

    def extend(self, items):
        return list.extend(self, self._refs(items))

    def __iadd__(self, other):
        return list.__iadd__(self, self._refs(other))

    def _refs(self, items):
        return map(self.ref, items)

    def _values(self, items):
        return map(self.value, items)

    def _sort_key(self, key=None):
        return self.value if key is None else lambda item: key(self.value(item))

    if sys.version_info < (3,):
        def sort(self, cmp=None, key=None, reverse=False):
            return list.sort(self, cmp=cmp, key=self._sort_key(key), reverse=reverse)

        def __setslice__(self, lower_bound, upper_bound, items):
            return self.__setitem__(slice(lower_bound, upper_bound), items)

        def __getslice__(self, lower_bound, upper_bound):
            return self.__getitem__(slice(lower_bound, upper_bound))
    else:
        def sort(self, key=None, reverse=False):
            return list.sort(self, key=self._sort_key(key), reverse=reverse)

==> apace-0.0.2.tar.gz: apace-0.0.2/apace/utils.py <==
import collections


class Signal:
    """A callable signal class to which callbacks can be registered.

    When ever the signal is emitted all registered functions are called.

    :param signals: Signals which this signal gets registered to.
    :type signals: Signal, optional
    """

    def __init__(self, *signals):
        self.callbacks = set()
        """Functions called when the signal is emitted."""
        for signal in signals:
            signal.connect(self)

    def __call__(self, *args, **kwargs):
        """Emit signal and call registered functions."""
        for callback in self.callbacks:
            callback(*args, **kwargs)

    def __str__(self):
        return 'Signal'

    __repr__ = __str__

    def connect(self, callback):
        """Connect a callback to this signal.

        :param function callback: Function which gets called when the signal is emitted.
        """
        self.callbacks.add(callback)


class Flag:
    def __init__(self, initial_value, signals=None):
        self.value = initial_value
        for signal in signals:
            signal.connect(self.set_value(True))

    def __bool__(self):
        return self.value

    def set_value(self, value):
        self.value = value


class AmbiguousNameError(Exception):
    """Raised if multiple elements or cells have the same name.

    :param str name: The ambiguous name.
    """

    def __init__(self, name):
        super().__init__(f'The name "{name}" is ambiguous. Names must be unique!')


def flatten(iterable):
    for element in iterable:
        if isinstance(element, collections.Iterable) and not isinstance(element, (str, bytes)):
            yield from flatten(element)
        else:
            yield element


class AttrDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__dict__ = self


class Structure:
    pass


# based on https://github.com/apieum/weakreflist/blob/master/weakreflist/weakreflist.py
# modified
from weakref import ref, ReferenceType
import sys


def is_slice(index):
    return isinstance(index, slice)


class WeakList(list):

    def __init__(self, items=list()):
        list.__init__(self, self._refs(items))

    def value(self, item):
        return item() if isinstance(item, ReferenceType) else item

    def ref(self, item):
        try:
            item = ref(item, self.remove_all)
        finally:
            return item

    def __contains__(self, item):
        return list.__contains__(self, self.ref(item))

    def __getitem__(self, index):
        items = list.__getitem__(self, index)
        return type(self)(self._values(items)) if is_slice(index) else self.value(items)

    def __setitem__(self, index, item):
        items = self._refs(item) if is_slice(index) else self.ref(item)
        return list.__setitem__(self, index, items)

    def __iter__(self):
        return iter(self[index] for index in range(len(self)))

    def __reversed__(self):
        reversed_self = type(self)(self)
        reversed_self.reverse()
        return reversed_self

    # modified
    def clear(self):
        del self[:]

    def append(self, item):
        list.append(self, self.ref(item))

    def remove(self, item):
        return list.remove(self, self.ref(item))

    def remove_all(self, item):
        item = self.ref(item)
        while list.__contains__(self, item):
            list.remove(self, item)

    def index(self, item):
        return list.index(self, self.ref(item))

    def count(self, item):
        return list.count(self, self.ref(item))

    def pop(self, index=-1):
        return self.value(list.pop(self, self.ref(index)))

    def insert(self, index, item):
        return list.insert(self, index, self.ref(item))

    def extend(self, items):
        return list.extend(self, self._refs(items))

    def __iadd__(self, other):
        return list.__iadd__(self, self._refs(other))

    def _refs(self, items):
        return map(self.ref, items)

    def _values(self, items):
        return map(self.value, items)

    def _sort_key(self, key=None):
        return self.value if key is None else lambda item: key(self.value(item))

    if sys.version_info < (3,):
        def sort(self, cmp=None, key=None, reverse=False):
            return list.sort(self, cmp=cmp, key=self._sort_key(key), reverse=reverse)

        def __setslice__(self, lower_bound, upper_bound, items):
            return self.__setitem__(slice(lower_bound, upper_bound), items)

        def __getslice__(self, lower_bound, upper_bound):
            return self.__getitem__(slice(lower_bound, upper_bound))
    else:
        def sort(self, key=None, reverse=False):
            return list.sort(self, key=self._sort_key(key), reverse=reverse)
